
#ifndef __GABEDIT_PSICODEGLOBAL_H__
#define __GABEDIT_PSICODEGLOBAL_H__

GtkWidget* psicodeWin;
PsicodeMolecule psicodeMolecule;
PsicodeColorFore psicodeColorFore;
PsicodeColorBack psicodeColorBack;
PsicodeBasis psicodeBasis;
PsicodeGuessWaveFunction psicodeGuessWaveFunction;

PsicodeFunctional* functionals;
PsicodeStdFunctional* stdFunctionals;
gdouble* sumFunctionals;

#endif /* __GABEDIT_PSICODEGLOBAL_H__ */

