
#ifndef __GABEDIT_MOLCASGLOBAL_H__
#define __GABEDIT_MOLCASGLOBAL_H__

GtkWidget* molcasWin;
MolcasMolecule molcasMolecule;
MolcasOptimization molcasOptimization;
MolcasSolvation molcasSolvation;
MolcasScf molcasScf;
MolcasGridIT molcasGridIT;
MolcasColorFore molcasColorFore;
MolcasColorBack molcasColorBack;
MolcasSystemVariables molcasSystemVariables;

#endif /* __GABEDIT_MOLCASGLOBAL_H__ */

