
#ifndef __GABEDIT_DEMONGLOBAL_H__
#define __GABEDIT_DEMONGLOBAL_H__

GtkWidget* demonWin;
DeMonMolecule demonMolecule;
DeMonColorFore demonColorFore;
DeMonColorBack demonColorBack;
DeMonBasis demonBasis;
DeMonGuessWaveFunction demonGuessWaveFunction;

DeMonFunctional* functionals;
DeMonStdFunctional* stdFunctionals;
gdouble* sumFunctionals;

#endif /* __GABEDIT_DEMONGLOBAL_H__ */

