
#ifndef __GABEDIT_GAMESSGLOBAL_H__
#define __GABEDIT_GAMESSGLOBAL_H__

GtkWidget* gamessWin;
GamessMolecule gamessMolecule;
GamessColorFore gamessColorFore;
GamessColorBack gamessColorBack;
GamessBasis gamessBasis;

GamessFunctional* functionals;
GamessStdFunctional* stdFunctionals;
gdouble* sumFunctionals;

#endif /* __GABEDIT_GAMESSGLOBAL_H__ */

