
#ifndef __GABEDIT_ORCAGLOBAL_H__
#define __GABEDIT_ORCAGLOBAL_H__

GtkWidget* orcaWin;
OrcaMolecule orcaMolecule;
OrcaColorFore orcaColorFore;
OrcaColorBack orcaColorBack;
OrcaBasis orcaBasis;
OrcaGuessWaveFunction orcaGuessWaveFunction;

OrcaFunctional* functionals;
OrcaStdFunctional* stdFunctionals;
gdouble* sumFunctionals;

#endif /* __GABEDIT_ORCAGLOBAL_H__ */

