
#ifndef __GABEDIT_FIREFLYGLOBAL_H__
#define __GABEDIT_FIREFLYGLOBAL_H__

GtkWidget* fireflyWin;
FireFlyMolecule fireflyMolecule;
FireFlyColorFore fireflyColorFore;
FireFlyColorBack fireflyColorBack;
FireFlyBasis fireflyBasis;

FireFlyFunctional* functionals;
FireFlyStdFunctional* stdFunctionals;
gdouble* sumFunctionals;

#endif /* __GABEDIT_FIREFLYGLOBAL_H__ */

