
#ifndef __GABEDIT_QCHEMGLOBAL_H__
#define __GABEDIT_QCHEMGLOBAL_H__

GtkWidget* qchemWin;
QChemMolecule qchemMolecule;
QChemColorFore qchemColorFore;
QChemColorBack qchemColorBack;
QChemBasis qchemBasis;
QChemGuessWaveFunction qchemGuessWaveFunction;

QChemFunctional* functionals;
QChemStdFunctional* stdFunctionals;
gdouble* sumFunctionals;

#endif /* __GABEDIT_QCHEMGLOBAL_H__ */

